// Kevin Li - 5:19 PM - 6/14/20

import SwiftUI

class YearlyCalendarManager: ObservableObject, ConfigurationDirectAccess {

    @Published var currentPage: Int = 0

    weak var parent: ElegantCalendarManager?

    let configuration: CalendarConfiguration
    let years: [Date]

    var currentYear: Date {
        years[currentPage]
    }

    init(configuration: CalendarConfiguration, initialYear: Date? = nil) {
        self.configuration = configuration

        let years = configuration.calendar.generateDates(
            inside: DateInterval(start: configuration.startDate,
                                 end: configuration.endDate),
            matching: .firstDayOfEveryYear)

        self.years = configuration.ascending ? years : years.reversed()

        if let initialYear = initialYear {
            let page = calendar.yearsBetween(referenceDate, and: initialYear)
            currentPage = page
        }
    }

}

extension YearlyCalendarManager {

    func scrollBackToToday() {
        scrollToYear(Date())
    }

    func scrollToYear(_ year: Date) {
        if !calendar.isDate(currentYear, equalTo: year, toGranularity: .year) {
            let page = calendar.yearsBetween(referenceDate, and: year)
            currentPage = page
        }
    }

    func willDisplay(page: Int) {
        if currentPage != page {
            currentPage = page
        }
    }

    func monthTapped(_ month: Date) {
        parent?.scrollToMonthAndShowMonthlyView(month)
    }

}

extension YearlyCalendarManager {

    static let mock = YearlyCalendarManager(configuration: .mock)
    static let mockWithInitialYear = YearlyCalendarManager(configuration: .mock, initialYear: .daysFromToday(365))

}

protocol YearlyCalendarManagerDirectAccess: ConfigurationDirectAccess, ElegantCalendarDirectAccess {

    var calendarManager: YearlyCalendarManager { get }
    var configuration: CalendarConfiguration { get }
    var parent: ElegantCalendarManager? { get }

}

extension YearlyCalendarManagerDirectAccess {

    var configuration: CalendarConfiguration {
        calendarManager.configuration
    }

    var parent: ElegantCalendarManager? {
        calendarManager.parent
    }

    var currentYear: Date {
        calendarManager.currentYear
    }

    var years: [Date] {
        calendarManager.years
    }

}

private extension Calendar {

    func yearsBetween(_ date1: Date, and date2: Date) -> Int {
        let startOfYearForDate1 = startOfYear(for: date1)
        let startOfYearForDate2 = startOfYear(for: date2)
        return abs(dateComponents([.year],
                              from: startOfYearForDate1,
                              to: startOfYearForDate2).year!)
    }

}
